/* THIS FILE IS AUTOGENERATED BY Codegen.py - DO NOT EDIT */

#ifndef mozilla_dom_GeneratedAtomList_h
#define mozilla_dom_GeneratedAtomList_h

#include "mozilla/dom/BindingUtils.h"

namespace mozilla {
namespace dom {

struct APZBucketAtoms
{
  InternedStringId scrollFrames_id;
  InternedStringId sequenceNumber_id;
};

struct APZTestDataAtoms
{
  InternedStringId paints_id;
  InternedStringId repaintRequests_id;
};

struct ActivityOptionsAtoms
{
  InternedStringId data_id;
  InternedStringId getFilterResults_id;
  InternedStringId name_id;
};

struct ActivityRequestHandlerAtoms
{
  InternedStringId postResult_id;
  InternedStringId postError_id;
  InternedStringId source_id;
  InternedStringId __init_id;
};

struct AdoptDownloadDictAtoms
{
  InternedStringId contentType_id;
  InternedStringId startTime_id;
  InternedStringId storageName_id;
  InternedStringId storagePath_id;
  InternedStringId url_id;
};

struct AesCbcParamsAtoms
{
  InternedStringId iv_id;
};

struct AesCtrParamsAtoms
{
  InternedStringId counter_id;
  InternedStringId length_id;
};

struct AesDerivedKeyParamsAtoms
{
  InternedStringId length_id;
};

struct AesGcmParamsAtoms
{
  InternedStringId additionalData_id;
  InternedStringId iv_id;
  InternedStringId tagLength_id;
};

struct AesKeyAlgorithmAtoms
{
  InternedStringId length_id;
};

struct AesKeyGenParamsAtoms
{
  InternedStringId length_id;
};

struct AlarmsManagerAtoms
{
  InternedStringId getAll_id;
  InternedStringId add_id;
  InternedStringId remove_id;
};

struct AlgorithmAtoms
{
  InternedStringId name_id;
};

struct AnimationEventInitAtoms
{
  InternedStringId animationName_id;
  InternedStringId elapsedTime_id;
  InternedStringId pseudoElement_id;
};

struct AppInfoAtoms
{
  InternedStringId isCoreApp_id;
  InternedStringId path_id;
};

struct AppNotificationServiceOptionsAtoms
{
  InternedStringId data_id;
  InternedStringId dbId_id;
  InternedStringId dir_id;
  InternedStringId id_id;
  InternedStringId lang_id;
  InternedStringId manifestURL_id;
  InternedStringId mozbehavior_id;
  InternedStringId tag_id;
  InternedStringId textClickable_id;
};

struct ArchiveReaderOptionsAtoms
{
  InternedStringId encoding_id;
};

struct AsyncScrollEventDetailAtoms
{
  InternedStringId height_id;
  InternedStringId left_id;
  InternedStringId scrollHeight_id;
  InternedStringId scrollWidth_id;
  InternedStringId top_id;
  InternedStringId width_id;
};

struct AutocompleteErrorEventInitAtoms
{
  InternedStringId reason_id;
};

struct AutocompleteInfoAtoms
{
  InternedStringId addressType_id;
  InternedStringId contactType_id;
  InternedStringId fieldName_id;
  InternedStringId section_id;
};

struct BeforeAfterKeyboardEventInitAtoms
{
  InternedStringId embeddedCancelled_id;
};

struct BlobEventInitAtoms
{
  InternedStringId data_id;
};

struct BlobPropertyBagAtoms
{
  InternedStringId endings_id;
  InternedStringId type_id;
};

struct BoxQuadOptionsAtoms
{
  InternedStringId box_id;
  InternedStringId relativeTo_id;
};

struct BrowserElementDownloadOptionsAtoms
{
  InternedStringId filename_id;
  InternedStringId referrer_id;
};

struct BrowserFeedWriterAtoms
{
  InternedStringId writeContent_id;
  InternedStringId close_id;
  InternedStringId __init_id;
};

struct CFStateChangeEventInitAtoms
{
  InternedStringId action_id;
  InternedStringId number_id;
  InternedStringId reason_id;
  InternedStringId serviceClass_id;
  InternedStringId timeSeconds_id;
};

struct CSPAtoms
{
  InternedStringId base_uri_id;
  InternedStringId connect_src_id;
  InternedStringId default_src_id;
  InternedStringId font_src_id;
  InternedStringId form_action_id;
  InternedStringId frame_ancestors_id;
  InternedStringId frame_src_id;
  InternedStringId img_src_id;
  InternedStringId manifest_src_id;
  InternedStringId media_src_id;
  InternedStringId object_src_id;
  InternedStringId referrer_id;
  InternedStringId report_only_id;
  InternedStringId report_uri_id;
  InternedStringId script_src_id;
  InternedStringId style_src_id;
};

struct CSPPoliciesAtoms
{
  InternedStringId csp_policies_id;
};

struct CSPReportAtoms
{
  InternedStringId csp_report_id;
};

struct CSPReportPropertiesAtoms
{
  InternedStringId blocked_uri_id;
  InternedStringId document_uri_id;
  InternedStringId line_number_id;
  InternedStringId original_policy_id;
  InternedStringId referrer_id;
  InternedStringId script_sample_id;
  InternedStringId source_file_id;
  InternedStringId violated_directive_id;
};

struct CSSTokenAtoms
{
  InternedStringId endOffset_id;
  InternedStringId hasSign_id;
  InternedStringId isInteger_id;
  InternedStringId number_id;
  InternedStringId startOffset_id;
  InternedStringId text_id;
  InternedStringId tokenType_id;
};

struct CacheBatchOperationAtoms
{
  InternedStringId options_id;
  InternedStringId request_id;
  InternedStringId response_id;
  InternedStringId type_id;
};

struct CacheQueryOptionsAtoms
{
  InternedStringId cacheName_id;
  InternedStringId ignoreMethod_id;
  InternedStringId ignoreSearch_id;
  InternedStringId ignoreVary_id;
};

struct CallEventInitAtoms
{
  InternedStringId call_id;
};

struct CallGroupErrorEventInitAtoms
{
  InternedStringId message_id;
  InternedStringId name_id;
};

struct CameraClosedEventInitAtoms
{
  InternedStringId reason_id;
};

struct CameraConfigurationAtoms
{
  InternedStringId mode_id;
  InternedStringId pictureSize_id;
  InternedStringId previewSize_id;
  InternedStringId recorderProfile_id;
};

struct CameraConfigurationEventInitAtoms
{
  InternedStringId mode_id;
  InternedStringId pictureSize_id;
  InternedStringId previewSize_id;
  InternedStringId recorderProfile_id;
};

struct CameraDetectedFaceInitAtoms
{
  InternedStringId bounds_id;
  InternedStringId hasLeftEye_id;
  InternedStringId hasMouth_id;
  InternedStringId hasRightEye_id;
  InternedStringId id_id;
  InternedStringId leftEye_id;
  InternedStringId mouth_id;
  InternedStringId rightEye_id;
  InternedStringId score_id;
};

struct CameraFacesDetectedEventInitAtoms
{
  InternedStringId faces_id;
};

struct CameraGetPromiseDataAtoms
{
  InternedStringId camera_id;
  InternedStringId configuration_id;
};

struct CameraPictureOptionsAtoms
{
  InternedStringId dateTime_id;
  InternedStringId fileFormat_id;
  InternedStringId pictureSize_id;
  InternedStringId position_id;
  InternedStringId rotation_id;
};

struct CameraPositionAtoms
{
  InternedStringId altitude_id;
  InternedStringId latitude_id;
  InternedStringId longitude_id;
  InternedStringId timestamp_id;
};

struct CameraRegionAtoms
{
  InternedStringId bottom_id;
  InternedStringId left_id;
  InternedStringId right_id;
  InternedStringId top_id;
  InternedStringId weight_id;
};

struct CameraSizeAtoms
{
  InternedStringId height_id;
  InternedStringId width_id;
};

struct CameraStartRecordingOptionsAtoms
{
  InternedStringId autoEnableLowLightTorch_id;
  InternedStringId maxFileSizeBytes_id;
  InternedStringId maxVideoLengthMs_id;
  InternedStringId posterFilepath_id;
  InternedStringId posterStorageArea_id;
  InternedStringId rotation_id;
};

struct CameraStateChangeEventInitAtoms
{
  InternedStringId newState_id;
};

struct CaretStateChangedEventInitAtoms
{
  InternedStringId boundingClientRect_id;
  InternedStringId caretVisible_id;
  InternedStringId collapsed_id;
  InternedStringId reason_id;
  InternedStringId selectionVisible_id;
};

struct ChromeFilePropertyBagAtoms
{
  InternedStringId name_id;
  InternedStringId temporary_id;
};

struct ChromeNotificationsAtoms
{
  InternedStringId mozResendAllNotifications_id;
};

struct ClientQueryOptionsAtoms
{
  InternedStringId includeUncontrolled_id;
  InternedStringId type_id;
};

struct ClipboardEventInitAtoms
{
  InternedStringId data_id;
  InternedStringId dataType_id;
};

struct CloseEventInitAtoms
{
  InternedStringId code_id;
  InternedStringId reason_id;
  InternedStringId wasClean_id;
};

struct CompositionClauseParametersAtoms
{
  InternedStringId length_id;
  InternedStringId selectionType_id;
};

struct ConnStatusDictAtoms
{
  InternedStringId status_id;
};

struct ConsoleCounterAtoms
{
  InternedStringId count_id;
  InternedStringId label_id;
};

struct ConsoleCounterErrorAtoms
{
  InternedStringId error_id;
};

struct ConsoleEventAtoms
{
  InternedStringId ID_id;
  InternedStringId arguments_id;
  InternedStringId columnNumber_id;
  InternedStringId counter_id;
  InternedStringId filename_id;
  InternedStringId functionName_id;
  InternedStringId groupName_id;
  InternedStringId innerID_id;
  InternedStringId level_id;
  InternedStringId lineNumber_id;
  InternedStringId private_id;
  InternedStringId styles_id;
  InternedStringId timeStamp_id;
  InternedStringId timer_id;
};

struct ConsoleProfileEventAtoms
{
  InternedStringId action_id;
  InternedStringId arguments_id;
};

struct ConsoleStackEntryAtoms
{
  InternedStringId columnNumber_id;
  InternedStringId filename_id;
  InternedStringId functionName_id;
  InternedStringId language_id;
  InternedStringId lineNumber_id;
};

struct ConsoleTimerEndAtoms
{
  InternedStringId duration_id;
  InternedStringId name_id;
};

struct ConsoleTimerErrorAtoms
{
  InternedStringId error_id;
};

struct ConsoleTimerStartAtoms
{
  InternedStringId name_id;
  InternedStringId started_id;
};

struct ConstrainBooleanParametersAtoms
{
  InternedStringId exact_id;
  InternedStringId ideal_id;
};

struct ConstrainDOMStringParametersAtoms
{
  InternedStringId exact_id;
  InternedStringId ideal_id;
};

struct ConstrainDoubleRangeAtoms
{
  InternedStringId exact_id;
  InternedStringId ideal_id;
  InternedStringId max_id;
  InternedStringId min_id;
};

struct ConstrainLongRangeAtoms
{
  InternedStringId exact_id;
  InternedStringId ideal_id;
  InternedStringId max_id;
  InternedStringId min_id;
};

struct ContactAddressAtoms
{
  InternedStringId countryName_id;
  InternedStringId locality_id;
  InternedStringId postalCode_id;
  InternedStringId pref_id;
  InternedStringId region_id;
  InternedStringId streetAddress_id;
  InternedStringId type_id;
};

struct ContactFieldAtoms
{
  InternedStringId pref_id;
  InternedStringId type_id;
  InternedStringId value_id;
};

struct ContactFindOptionsAtoms
{
  InternedStringId filterBy_id;
  InternedStringId filterLimit_id;
  InternedStringId filterOp_id;
  InternedStringId filterValue_id;
};

struct ContactFindSortOptionsAtoms
{
  InternedStringId sortBy_id;
  InternedStringId sortOrder_id;
};

struct ContactManagerAtoms
{
  InternedStringId find_id;
  InternedStringId getAll_id;
  InternedStringId clear_id;
  InternedStringId save_id;
  InternedStringId remove_id;
  InternedStringId getRevision_id;
  InternedStringId getCount_id;
  InternedStringId oncontactchange_id;
};

struct ContactPropertiesAtoms
{
  InternedStringId additionalName_id;
  InternedStringId adr_id;
  InternedStringId anniversary_id;
  InternedStringId bday_id;
  InternedStringId category_id;
  InternedStringId email_id;
  InternedStringId familyName_id;
  InternedStringId genderIdentity_id;
  InternedStringId givenName_id;
  InternedStringId honorificPrefix_id;
  InternedStringId honorificSuffix_id;
  InternedStringId impp_id;
  InternedStringId jobTitle_id;
  InternedStringId key_id;
  InternedStringId name_id;
  InternedStringId nickname_id;
  InternedStringId note_id;
  InternedStringId org_id;
  InternedStringId phoneticFamilyName_id;
  InternedStringId phoneticGivenName_id;
  InternedStringId photo_id;
  InternedStringId sex_id;
  InternedStringId tel_id;
  InternedStringId url_id;
};

struct ContactTelFieldAtoms
{
  InternedStringId carrier_id;
};

struct ContextAttributes2DAtoms
{
  InternedStringId alpha_id;
  InternedStringId willReadFrequently_id;
};

struct ConvertCoordinateOptionsAtoms
{
  InternedStringId fromBox_id;
  InternedStringId toBox_id;
};

struct CreateFileOptionsAtoms
{
  InternedStringId data_id;
  InternedStringId ifExists_id;
};

struct CryptoKeyPairAtoms
{
  InternedStringId privateKey_id;
  InternedStringId publicKey_id;
};

struct CustomEventInitAtoms
{
  InternedStringId detail_id;
};

struct DNSCacheDictAtoms
{
  InternedStringId entries_id;
};

struct DNSLookupDictAtoms
{
  InternedStringId address_id;
  InternedStringId answer_id;
  InternedStringId error_id;
};

struct DOMApplicationAtoms
{
  InternedStringId manifest_id;
  InternedStringId updateManifest_id;
  InternedStringId manifestURL_id;
  InternedStringId origin_id;
  InternedStringId installOrigin_id;
  InternedStringId installTime_id;
  InternedStringId removable_id;
  InternedStringId enabled_id;
  InternedStringId receipts_id;
  InternedStringId progress_id;
  InternedStringId installState_id;
  InternedStringId lastUpdateCheck_id;
  InternedStringId updateTime_id;
  InternedStringId downloadAvailable_id;
  InternedStringId downloading_id;
  InternedStringId readyToApplyDownload_id;
  InternedStringId downloadSize_id;
  InternedStringId downloadError_id;
  InternedStringId onprogress_id;
  InternedStringId ondownloadsuccess_id;
  InternedStringId ondownloaderror_id;
  InternedStringId ondownloadavailable_id;
  InternedStringId ondownloadapplied_id;
  InternedStringId download_id;
  InternedStringId cancelDownload_id;
  InternedStringId launch_id;
  InternedStringId clearBrowserData_id;
  InternedStringId checkForUpdate_id;
  InternedStringId connect_id;
  InternedStringId getConnections_id;
  InternedStringId addReceipt_id;
  InternedStringId removeReceipt_id;
  InternedStringId replaceReceipt_id;
  InternedStringId export_id;
  InternedStringId getLocalizedValue_id;
};

struct DOMApplicationsManagerAtoms
{
  InternedStringId getAll_id;
  InternedStringId getNotInstalled_id;
  InternedStringId applyDownload_id;
  InternedStringId uninstall_id;
  InternedStringId import_id;
  InternedStringId extractManifest_id;
  InternedStringId setEnabled_id;
  InternedStringId getIcon_id;
  InternedStringId oninstall_id;
  InternedStringId onuninstall_id;
  InternedStringId onenabledstatechange_id;
};

struct DOMApplicationsRegistryAtoms
{
  InternedStringId mgmt_id;
  InternedStringId install_id;
  InternedStringId installPackage_id;
  InternedStringId getSelf_id;
  InternedStringId getInstalled_id;
  InternedStringId checkInstalled_id;
  InternedStringId getAdditionalLanguages_id;
  InternedStringId getLocalizationResource_id;
};

struct DOMDownloadAtoms
{
  InternedStringId totalBytes_id;
  InternedStringId currentBytes_id;
  InternedStringId url_id;
  InternedStringId path_id;
  InternedStringId storageName_id;
  InternedStringId storagePath_id;
  InternedStringId state_id;
  InternedStringId contentType_id;
  InternedStringId startTime_id;
  InternedStringId id_id;
  InternedStringId sourceAppManifestURL_id;
  InternedStringId error_id;
  InternedStringId pause_id;
  InternedStringId resume_id;
  InternedStringId onstatechange_id;
};

struct DOMDownloadManagerAtoms
{
  InternedStringId getDownloads_id;
  InternedStringId remove_id;
  InternedStringId clearAllDone_id;
  InternedStringId adoptDownload_id;
  InternedStringId ondownloadstart_id;
};

struct DOMPointInitAtoms
{
  InternedStringId w_id;
  InternedStringId x_id;
  InternedStringId y_id;
  InternedStringId z_id;
};

struct DOMTransactionAtoms
{
  InternedStringId label_id;
  InternedStringId executeAutomatic_id;
  InternedStringId execute_id;
  InternedStringId undo_id;
  InternedStringId redo_id;
};

struct DOMTransactionEventInitAtoms
{
  InternedStringId transactions_id;
};

struct DOMWindowResizeEventDetailAtoms
{
  InternedStringId height_id;
  InternedStringId width_id;
};

struct DataErrorEventInitAtoms
{
  InternedStringId message_id;
};

struct DataStoreChangeEventInitAtoms
{
  InternedStringId id_id;
  InternedStringId operation_id;
  InternedStringId owner_id;
  InternedStringId revisionId_id;
};

struct DataStoreCursorImplAtoms
{
  InternedStringId store_id;
  InternedStringId next_id;
  InternedStringId close_id;
};

struct DataStoreImplAtoms
{
  InternedStringId setEventTarget_id;
  InternedStringId name_id;
  InternedStringId owner_id;
  InternedStringId readOnly_id;
  InternedStringId get_id;
  InternedStringId put_id;
  InternedStringId add_id;
  InternedStringId remove_id;
  InternedStringId clear_id;
  InternedStringId revisionId_id;
  InternedStringId getLength_id;
  InternedStringId sync_id;
};

struct DataStoreRevisionDataAtoms
{
  InternedStringId objectId_id;
  InternedStringId operation_id;
  InternedStringId revisionId_id;
};

struct DataStoreTaskAtoms
{
  InternedStringId data_id;
  InternedStringId id_id;
  InternedStringId operation_id;
  InternedStringId revisionId_id;
};

struct DeprecatedRTCOfferOptionsSetAtoms
{
  InternedStringId MozBundleOnly_id;
  InternedStringId MozDontOfferDataChannel_id;
  InternedStringId OfferToReceiveAudio_id;
  InternedStringId OfferToReceiveVideo_id;
};

struct DeviceAccelerationInitAtoms
{
  InternedStringId x_id;
  InternedStringId y_id;
  InternedStringId z_id;
};

struct DeviceLightEventInitAtoms
{
  InternedStringId value_id;
};

struct DeviceMotionEventInitAtoms
{
  InternedStringId acceleration_id;
  InternedStringId accelerationIncludingGravity_id;
  InternedStringId interval_id;
  InternedStringId rotationRate_id;
};

struct DeviceOrientationEventInitAtoms
{
  InternedStringId absolute_id;
  InternedStringId alpha_id;
  InternedStringId beta_id;
  InternedStringId gamma_id;
};

struct DeviceProximityEventInitAtoms
{
  InternedStringId max_id;
  InternedStringId min_id;
  InternedStringId value_id;
};

struct DeviceRotationRateInitAtoms
{
  InternedStringId alpha_id;
  InternedStringId beta_id;
  InternedStringId gamma_id;
};

struct DeviceStorageAreaChangedEventInitAtoms
{
  InternedStringId operation_id;
  InternedStringId storageName_id;
};

struct DeviceStorageChangeEventInitAtoms
{
  InternedStringId path_id;
  InternedStringId reason_id;
};

struct DeviceStorageEnumerationParametersAtoms
{
  InternedStringId since_id;
};

struct DhImportKeyParamsAtoms
{
  InternedStringId generator_id;
  InternedStringId prime_id;
};

struct DhKeyAlgorithmAtoms
{
  InternedStringId generator_id;
  InternedStringId prime_id;
};

struct DhKeyDeriveParamsAtoms
{
  InternedStringId public_id;
};

struct DhKeyGenParamsAtoms
{
  InternedStringId generator_id;
  InternedStringId prime_id;
};

struct DnsCacheEntryAtoms
{
  InternedStringId expiration_id;
  InternedStringId family_id;
  InternedStringId hostaddr_id;
  InternedStringId hostname_id;
};

struct DownloadEventInitAtoms
{
  InternedStringId download_id;
};

struct DragEventInitAtoms
{
  InternedStringId dataTransfer_id;
};

struct EcKeyAlgorithmAtoms
{
  InternedStringId namedCurve_id;
};

struct EcKeyGenParamsAtoms
{
  InternedStringId namedCurve_id;
};

struct EcKeyImportParamsAtoms
{
  InternedStringId namedCurve_id;
};

struct EcdhKeyDeriveParamsAtoms
{
  InternedStringId public_id;
};

struct EcdsaParamsAtoms
{
  InternedStringId hash_id;
};

struct ElementRegistrationOptionsAtoms
{
  InternedStringId extends_id;
  InternedStringId prototype_id;
};

struct EngineeringModeAtoms
{
  InternedStringId getValue_id;
  InternedStringId setValue_id;
  InternedStringId onmessage_id;
};

struct ErrorEventInitAtoms
{
  InternedStringId colno_id;
  InternedStringId error_id;
  InternedStringId filename_id;
  InternedStringId lineno_id;
  InternedStringId message_id;
};

struct EventInitAtoms
{
  InternedStringId bubbles_id;
  InternedStringId cancelable_id;
};

struct EventListenerAtoms
{
  InternedStringId handleEvent_id;
};

struct EventModifierInitAtoms
{
  InternedStringId altKey_id;
  InternedStringId ctrlKey_id;
  InternedStringId metaKey_id;
  InternedStringId modifierAltGraph_id;
  InternedStringId modifierCapsLock_id;
  InternedStringId modifierFn_id;
  InternedStringId modifierFnLock_id;
  InternedStringId modifierNumLock_id;
  InternedStringId modifierOS_id;
  InternedStringId modifierScrollLock_id;
  InternedStringId modifierSymbol_id;
  InternedStringId modifierSymbolLock_id;
  InternedStringId shiftKey_id;
};

struct EventSourceInitAtoms
{
  InternedStringId withCredentials_id;
};

struct ExternalAtoms
{
  InternedStringId AddSearchProvider_id;
  InternedStringId IsSearchProviderInstalled_id;
  InternedStringId addSearchEngine_id;
};

struct FetchEventInitAtoms
{
  InternedStringId client_id;
  InternedStringId isReload_id;
  InternedStringId request_id;
};

struct FilePropertyBagAtoms
{
  InternedStringId lastModified_id;
  InternedStringId type_id;
};

struct FocusEventInitAtoms
{
  InternedStringId relatedTarget_id;
};

struct FontFaceDescriptorsAtoms
{
  InternedStringId featureSettings_id;
  InternedStringId stretch_id;
  InternedStringId style_id;
  InternedStringId unicodeRange_id;
  InternedStringId variant_id;
  InternedStringId weight_id;
};

struct FontFaceSetIteratorResultAtoms
{
  InternedStringId done_id;
  InternedStringId value_id;
};

struct FontFaceSetLoadEventInitAtoms
{
  InternedStringId fontfaces_id;
};

struct FrameUniformityAtoms
{
  InternedStringId frameUniformity_id;
  InternedStringId layerAddress_id;
};

struct FrameUniformityResultsAtoms
{
  InternedStringId layerUniformities_id;
};

struct GamepadAxisMoveEventInitAtoms
{
  InternedStringId axis_id;
  InternedStringId value_id;
};

struct GamepadButtonEventInitAtoms
{
  InternedStringId button_id;
};

struct GamepadEventInitAtoms
{
  InternedStringId gamepad_id;
};

struct GetNotificationOptionsAtoms
{
  InternedStringId tag_id;
};

struct HalfOpenInfoDictAtoms
{
  InternedStringId speculative_id;
};

struct HashChangeEventInitAtoms
{
  InternedStringId newURL_id;
  InternedStringId oldURL_id;
};

struct HeapSnapshotBoundariesAtoms
{
  InternedStringId debugger_id;
  InternedStringId globals_id;
  InternedStringId runtime_id;
};

struct HitRegionOptionsAtoms
{
  InternedStringId control_id;
  InternedStringId id_id;
  InternedStringId path_id;
};

struct HmacDerivedKeyParamsAtoms
{
  InternedStringId length_id;
};

struct HmacImportParamsAtoms
{
  InternedStringId hash_id;
};

struct HmacKeyAlgorithmAtoms
{
  InternedStringId hash_id;
  InternedStringId length_id;
};

struct HmacKeyGenParamsAtoms
{
  InternedStringId hash_id;
  InternedStringId length_id;
};

struct HttpConnDictAtoms
{
  InternedStringId connections_id;
};

struct HttpConnInfoAtoms
{
  InternedStringId protocolVersion_id;
  InternedStringId rtt_id;
  InternedStringId ttl_id;
};

struct HttpConnectionElementAtoms
{
  InternedStringId active_id;
  InternedStringId halfOpens_id;
  InternedStringId host_id;
  InternedStringId idle_id;
  InternedStringId port_id;
  InternedStringId spdy_id;
  InternedStringId ssl_id;
};

struct IDBFileMetadataParametersAtoms
{
  InternedStringId lastModified_id;
  InternedStringId size_id;
};

struct IDBIndexParametersAtoms
{
  InternedStringId multiEntry_id;
  InternedStringId unique_id;
};

struct IDBObjectStoreParametersAtoms
{
  InternedStringId autoIncrement_id;
  InternedStringId keyPath_id;
};

struct IDBOpenDBOptionsAtoms
{
  InternedStringId storage_id;
  InternedStringId version_id;
};

struct IDBVersionChangeEventInitAtoms
{
  InternedStringId newVersion_id;
  InternedStringId oldVersion_id;
};

struct IccCardLockRetryCountAtoms
{
  InternedStringId retryCount_id;
};

struct IccCardLockStatusAtoms
{
  InternedStringId enabled_id;
};

struct IccChangeEventInitAtoms
{
  InternedStringId iccId_id;
};

struct IccSetCardLockOptionsAtoms
{
  InternedStringId enabled_id;
  InternedStringId lockType_id;
  InternedStringId newPin_id;
  InternedStringId pin_id;
  InternedStringId pin2_id;
};

struct IccUnlockCardLockOptionsAtoms
{
  InternedStringId lockType_id;
  InternedStringId newPin_id;
  InternedStringId pin_id;
  InternedStringId puk_id;
};

struct ImageCaptureErrorEventInitAtoms
{
  InternedStringId imageCaptureError_id;
};

struct InputEventInitAtoms
{
  InternedStringId isComposing_id;
};

struct InspectorRGBATupleAtoms
{
  InternedStringId a_id;
  InternedStringId b_id;
  InternedStringId g_id;
  InternedStringId r_id;
};

struct InspectorRGBTripleAtoms
{
  InternedStringId b_id;
  InternedStringId g_id;
  InternedStringId r_id;
};

struct InstallParametersAtoms
{
  InternedStringId categories_id;
  InternedStringId receipts_id;
};

struct InstallTriggerDataAtoms
{
  InternedStringId Hash_id;
  InternedStringId IconURL_id;
  InternedStringId URL_id;
};

struct InstallTriggerImplAtoms
{
  InternedStringId enabled_id;
  InternedStringId updateEnabled_id;
  InternedStringId install_id;
  InternedStringId installChrome_id;
  InternedStringId startSoftwareUpdate_id;
};

struct JsonWebKeyAtoms
{
  InternedStringId alg_id;
  InternedStringId crv_id;
  InternedStringId d_id;
  InternedStringId dp_id;
  InternedStringId dq_id;
  InternedStringId e_id;
  InternedStringId ext_id;
  InternedStringId k_id;
  InternedStringId key_ops_id;
  InternedStringId kty_id;
  InternedStringId n_id;
  InternedStringId oth_id;
  InternedStringId p_id;
  InternedStringId q_id;
  InternedStringId qi_id;
  InternedStringId use_id;
  InternedStringId x_id;
  InternedStringId y_id;
};

struct KeyAlgorithmAtoms
{
  InternedStringId name_id;
};

struct KeyboardEventInitAtoms
{
  InternedStringId charCode_id;
  InternedStringId code_id;
  InternedStringId isComposing_id;
  InternedStringId key_id;
  InternedStringId keyCode_id;
  InternedStringId location_id;
  InternedStringId repeat_id;
  InternedStringId which_id;
};

struct LanguageDescAtoms
{
  InternedStringId name_id;
  InternedStringId revision_id;
  InternedStringId target_id;
};

struct LifecycleCallbacksAtoms
{
  InternedStringId attachedCallback_id;
  InternedStringId attributeChangedCallback_id;
  InternedStringId createdCallback_id;
  InternedStringId detachedCallback_id;
};

struct MediaKeyMessageEventInitAtoms
{
  InternedStringId message_id;
  InternedStringId messageType_id;
};

struct MediaKeyNeededEventInitAtoms
{
  InternedStringId initData_id;
  InternedStringId initDataType_id;
};

struct MediaKeySystemOptionsAtoms
{
  InternedStringId audioCapability_id;
  InternedStringId audioType_id;
  InternedStringId initDataType_id;
  InternedStringId stateful_id;
  InternedStringId uniqueidentifier_id;
  InternedStringId videoCapability_id;
  InternedStringId videoType_id;
};

struct MediaRecorderOptionsAtoms
{
  InternedStringId mimeType_id;
};

struct MediaStreamConstraintsAtoms
{
  InternedStringId audio_id;
  InternedStringId fake_id;
  InternedStringId fakeTracks_id;
  InternedStringId peerIdentity_id;
  InternedStringId picture_id;
  InternedStringId video_id;
};

struct MediaStreamEventInitAtoms
{
  InternedStringId stream_id;
};

struct MediaStreamTrackEventInitAtoms
{
  InternedStringId receiver_id;
  InternedStringId stream_id;
  InternedStringId track_id;
};

struct MediaTrackConstraintSetAtoms
{
  InternedStringId browserWindow_id;
  InternedStringId facingMode_id;
  InternedStringId frameRate_id;
  InternedStringId height_id;
  InternedStringId mediaSource_id;
  InternedStringId scrollWithPage_id;
  InternedStringId width_id;
};

struct MediaTrackConstraintsAtoms
{
  InternedStringId advanced_id;
};

struct MessageEventInitAtoms
{
  InternedStringId data_id;
  InternedStringId lastEventId_id;
  InternedStringId origin_id;
  InternedStringId ports_id;
  InternedStringId source_id;
};

struct MmsAttachmentAtoms
{
  InternedStringId content_id;
  InternedStringId id_id;
  InternedStringId location_id;
};

struct MmsDeliveryInfoAtoms
{
  InternedStringId deliveryStatus_id;
  InternedStringId deliveryTimestamp_id;
  InternedStringId readStatus_id;
  InternedStringId readTimestamp_id;
  InternedStringId receiver_id;
};

struct MmsParametersAtoms
{
  InternedStringId attachments_id;
  InternedStringId receivers_id;
  InternedStringId smil_id;
  InternedStringId subject_id;
};

struct MmsSendParametersAtoms
{
  InternedStringId serviceId_id;
};

struct MobileMessageFilterAtoms
{
  InternedStringId delivery_id;
  InternedStringId endDate_id;
  InternedStringId numbers_id;
  InternedStringId read_id;
  InternedStringId startDate_id;
  InternedStringId threadId_id;
};

struct MouseEventInitAtoms
{
  InternedStringId button_id;
  InternedStringId buttons_id;
  InternedStringId clientX_id;
  InternedStringId clientY_id;
  InternedStringId movementX_id;
  InternedStringId movementY_id;
  InternedStringId relatedTarget_id;
  InternedStringId screenX_id;
  InternedStringId screenY_id;
};

struct MozApplicationEventInitAtoms
{
  InternedStringId application_id;
};

struct MozCallBarringOptionsAtoms
{
  InternedStringId enabled_id;
  InternedStringId newPin_id;
  InternedStringId password_id;
  InternedStringId pin_id;
  InternedStringId program_id;
  InternedStringId serviceClass_id;
};

struct MozCallForwardingOptionsAtoms
{
  InternedStringId action_id;
  InternedStringId active_id;
  InternedStringId number_id;
  InternedStringId reason_id;
  InternedStringId serviceClass_id;
  InternedStringId timeSeconds_id;
};

struct MozCellBroadcastEventInitAtoms
{
  InternedStringId message_id;
};

struct MozClirModeEventInitAtoms
{
  InternedStringId mode_id;
};

struct MozClirStatusAtoms
{
  InternedStringId m_id;
  InternedStringId n_id;
};

struct MozContactChangeEventInitAtoms
{
  InternedStringId contactID_id;
  InternedStringId reason_id;
};

struct MozEmergencyCbModeEventInitAtoms
{
  InternedStringId active_id;
  InternedStringId timeoutMs_id;
};

struct MozIdleObserverAtoms
{
  InternedStringId time_id;
  InternedStringId onidle_id;
  InternedStringId onactive_id;
};

struct MozInputContextAtoms
{
  InternedStringId type_id;
  InternedStringId inputType_id;
  InternedStringId inputMode_id;
  InternedStringId lang_id;
  InternedStringId getText_id;
  InternedStringId selectionStart_id;
  InternedStringId selectionEnd_id;
  InternedStringId textBeforeCursor_id;
  InternedStringId textAfterCursor_id;
  InternedStringId setSelectionRange_id;
  InternedStringId onselectionchange_id;
  InternedStringId replaceSurroundingText_id;
  InternedStringId deleteSurroundingText_id;
  InternedStringId onsurroundingtextchange_id;
  InternedStringId sendKey_id;
  InternedStringId setComposition_id;
  InternedStringId endComposition_id;
};

struct MozInputMethodAtoms
{
  InternedStringId mgmt_id;
  InternedStringId oninputcontextchange_id;
  InternedStringId inputcontext_id;
  InternedStringId setActive_id;
  InternedStringId addInput_id;
  InternedStringId removeInput_id;
  InternedStringId setValue_id;
  InternedStringId setSelectedOption_id;
  InternedStringId setSelectedOptions_id;
  InternedStringId removeFocus_id;
};

struct MozInputMethodManagerAtoms
{
  InternedStringId showAll_id;
  InternedStringId next_id;
  InternedStringId supportsSwitching_id;
  InternedStringId hide_id;
};

struct MozInterAppConnectionAtoms
{
  InternedStringId keyword_id;
  InternedStringId publisher_id;
  InternedStringId subscriber_id;
  InternedStringId cancel_id;
  InternedStringId __init_id;
};

struct MozInterAppConnectionRequestAtoms
{
  InternedStringId keyword_id;
  InternedStringId port_id;
  InternedStringId __init_id;
};

struct MozInterAppMessageEventInitAtoms
{
  InternedStringId data_id;
};

struct MozInterAppMessagePortAtoms
{
  InternedStringId postMessage_id;
  InternedStringId start_id;
  InternedStringId close_id;
  InternedStringId onmessage_id;
  InternedStringId __init_id;
};

struct MozMMIResultAtoms
{
  InternedStringId additionalInformation_id;
  InternedStringId serviceCode_id;
  InternedStringId statusMessage_id;
  InternedStringId success_id;
};

struct MozMessageDeletedEventInitAtoms
{
  InternedStringId deletedMessageIds_id;
  InternedStringId deletedThreadIds_id;
};

struct MozMmsEventInitAtoms
{
  InternedStringId message_id;
};

struct MozOtaStatusEventInitAtoms
{
  InternedStringId status_id;
};

struct MozPluginParameterAtoms
{
  InternedStringId name_id;
  InternedStringId value_id;
};

struct MozSelfSupportAtoms
{
  InternedStringId healthReportDataSubmissionEnabled_id;
  InternedStringId getHealthReportPayload_id;
  InternedStringId getTelemetryPingList_id;
  InternedStringId getTelemetryPing_id;
  InternedStringId getCurrentTelemetryEnvironment_id;
  InternedStringId getCurrentTelemetrySubsessionPing_id;
  InternedStringId resetPref_id;
  InternedStringId resetSearchEngines_id;
  InternedStringId __init_id;
};

struct MozSettingsEventInitAtoms
{
  InternedStringId settingName_id;
  InternedStringId settingValue_id;
};

struct MozSettingsTransactionEventInitAtoms
{
  InternedStringId error_id;
};

struct MozSmsEventInitAtoms
{
  InternedStringId message_id;
};

struct MozStkBipMessageAtoms
{
  InternedStringId text_id;
};

struct MozStkBrowserSettingAtoms
{
  InternedStringId confirmMessage_id;
  InternedStringId mode_id;
  InternedStringId url_id;
};

struct MozStkBrowserTerminationEventAtoms
{
  InternedStringId eventType_id;
  InternedStringId terminationCause_id;
};

struct MozStkCallEventAtoms
{
  InternedStringId error_id;
  InternedStringId eventType_id;
  InternedStringId isIssuedByRemote_id;
  InternedStringId number_id;
};

struct MozStkCommandAtoms
{
  InternedStringId commandNumber_id;
  InternedStringId commandQualifier_id;
  InternedStringId options_id;
  InternedStringId typeOfCommand_id;
};

struct MozStkCommandEventInitAtoms
{
  InternedStringId command_id;
};

struct MozStkDurationAtoms
{
  InternedStringId timeInterval_id;
  InternedStringId timeUnit_id;
};

struct MozStkGeneralEventAtoms
{
  InternedStringId eventType_id;
};

struct MozStkIconAtoms
{
  InternedStringId codingScheme_id;
  InternedStringId height_id;
  InternedStringId pixels_id;
  InternedStringId width_id;
};

struct MozStkIconContainerAtoms
{
  InternedStringId iconSelfExplanatory_id;
  InternedStringId icons_id;
};

struct MozStkInputAtoms
{
  InternedStringId defaultText_id;
  InternedStringId duration_id;
  InternedStringId hideInput_id;
  InternedStringId isAlphabet_id;
  InternedStringId isHelpAvailable_id;
  InternedStringId isPacked_id;
  InternedStringId isUCS2_id;
  InternedStringId isYesNoRequested_id;
  InternedStringId maxLength_id;
  InternedStringId minLength_id;
  InternedStringId text_id;
};

struct MozStkItemAtoms
{
  InternedStringId identifier_id;
  InternedStringId text_id;
};

struct MozStkLanguageSelectionEventAtoms
{
  InternedStringId eventType_id;
  InternedStringId language_id;
};

struct MozStkLocalInfoAtoms
{
  InternedStringId date_id;
  InternedStringId imei_id;
  InternedStringId language_id;
  InternedStringId locationInfo_id;
};

struct MozStkLocationEventAtoms
{
  InternedStringId eventType_id;
  InternedStringId locationInfo_id;
  InternedStringId locationStatus_id;
};

struct MozStkLocationInfoAtoms
{
  InternedStringId gsmCellId_id;
  InternedStringId gsmLocationAreaCode_id;
  InternedStringId mcc_id;
  InternedStringId mnc_id;
};

struct MozStkMenuAtoms
{
  InternedStringId defaultItem_id;
  InternedStringId isHelpAvailable_id;
  InternedStringId items_id;
  InternedStringId nextActionList_id;
  InternedStringId presentationType_id;
  InternedStringId title_id;
};

struct MozStkPlayToneAtoms
{
  InternedStringId duration_id;
  InternedStringId isVibrate_id;
  InternedStringId text_id;
  InternedStringId tone_id;
};

struct MozStkProvideLocalInfoAtoms
{
  InternedStringId localInfoType_id;
};

struct MozStkResponseAtoms
{
  InternedStringId additionalInformation_id;
  InternedStringId hasConfirmed_id;
  InternedStringId input_id;
  InternedStringId isYesNo_id;
  InternedStringId itemIdentifier_id;
  InternedStringId localInfo_id;
  InternedStringId resultCode_id;
  InternedStringId timer_id;
};

struct MozStkSetUpCallAtoms
{
  InternedStringId address_id;
  InternedStringId callMessage_id;
  InternedStringId confirmMessage_id;
  InternedStringId duration_id;
};

struct MozStkSetUpEventListAtoms
{
  InternedStringId eventList_id;
};

struct MozStkTextMessageAtoms
{
  InternedStringId duration_id;
  InternedStringId isHighPriority_id;
  InternedStringId responseNeeded_id;
  InternedStringId text_id;
  InternedStringId userClear_id;
};

struct MozStkTimerAtoms
{
  InternedStringId timerAction_id;
  InternedStringId timerId_id;
  InternedStringId timerValue_id;
};

struct MozTetheringManagerAtoms
{
  InternedStringId setTetheringEnabled_id;
};

struct MozVoicemailEventInitAtoms
{
  InternedStringId status_id;
};

struct MozXMLHttpRequestParametersAtoms
{
  InternedStringId mozAnon_id;
  InternedStringId mozSystem_id;
};

struct MutationObserverInitAtoms
{
  InternedStringId animations_id;
  InternedStringId attributeFilter_id;
  InternedStringId attributeOldValue_id;
  InternedStringId attributes_id;
  InternedStringId characterData_id;
  InternedStringId characterDataOldValue_id;
  InternedStringId childList_id;
  InternedStringId subtree_id;
};

struct MutationObservingInfoAtoms
{
  InternedStringId observedNode_id;
};

struct NativeOSFileReadOptionsAtoms
{
  InternedStringId bytes_id;
  InternedStringId encoding_id;
};

struct NetworkCommandOptionsAtoms
{
  InternedStringId cmd_id;
  InternedStringId curExternalIfname_id;
  InternedStringId curInternalIfname_id;
  InternedStringId dns1_id;
  InternedStringId dns1_long_id;
  InternedStringId dns2_id;
  InternedStringId dns2_long_id;
  InternedStringId dnses_id;
  InternedStringId domain_id;
  InternedStringId enable_id;
  InternedStringId enabled_id;
  InternedStringId endIp_id;
  InternedStringId externalIfname_id;
  InternedStringId gateway_id;
  InternedStringId gateway_long_id;
  InternedStringId gateways_id;
  InternedStringId id_id;
  InternedStringId ifname_id;
  InternedStringId interfaceList_id;
  InternedStringId internalIfname_id;
  InternedStringId ip_id;
  InternedStringId ipaddr_id;
  InternedStringId key_id;
  InternedStringId link_id;
  InternedStringId mask_id;
  InternedStringId maskLength_id;
  InternedStringId mode_id;
  InternedStringId oldIfname_id;
  InternedStringId preExternalIfname_id;
  InternedStringId preInternalIfname_id;
  InternedStringId prefix_id;
  InternedStringId prefixLength_id;
  InternedStringId report_id;
  InternedStringId security_id;
  InternedStringId serverIp_id;
  InternedStringId ssid_id;
  InternedStringId startIp_id;
  InternedStringId threshold_id;
  InternedStringId usbEndIp_id;
  InternedStringId usbStartIp_id;
  InternedStringId wifiEndIp_id;
  InternedStringId wifiStartIp_id;
  InternedStringId wifictrlinterfacename_id;
};

struct NetworkResultOptionsAtoms
{
  InternedStringId broadcast_id;
  InternedStringId curExternalIfname_id;
  InternedStringId curInternalIfname_id;
  InternedStringId dns1_id;
  InternedStringId dns1_str_id;
  InternedStringId dns2_id;
  InternedStringId dns2_str_id;
  InternedStringId enable_id;
  InternedStringId error_id;
  InternedStringId gateway_id;
  InternedStringId gateway_str_id;
  InternedStringId id_id;
  InternedStringId ipaddr_id;
  InternedStringId ipaddr_str_id;
  InternedStringId lease_id;
  InternedStringId mask_id;
  InternedStringId mask_str_id;
  InternedStringId netId_id;
  InternedStringId reason_id;
  InternedStringId reply_id;
  InternedStringId result_id;
  InternedStringId resultCode_id;
  InternedStringId resultReason_id;
  InternedStringId ret_id;
  InternedStringId route_id;
  InternedStringId server_id;
  InternedStringId server_str_id;
  InternedStringId success_id;
  InternedStringId topic_id;
  InternedStringId vendor_str_id;
};

struct NetworkStatsDataAtoms
{
  InternedStringId receivedBytes_id;
  InternedStringId sentBytes_id;
  InternedStringId timestamp_id;
};

struct NodeFilterAtoms
{
  InternedStringId acceptNode_id;
};

struct NotificationBehaviorAtoms
{
  InternedStringId noclear_id;
  InternedStringId noscreen_id;
  InternedStringId showOnlyOnce_id;
  InternedStringId soundFile_id;
  InternedStringId vibrationPattern_id;
};

struct NotificationOptionsAtoms
{
  InternedStringId body_id;
  InternedStringId data_id;
  InternedStringId dir_id;
  InternedStringId icon_id;
  InternedStringId lang_id;
  InternedStringId mozbehavior_id;
  InternedStringId tag_id;
};

struct OpenWindowEventDetailAtoms
{
  InternedStringId features_id;
  InternedStringId frameElement_id;
  InternedStringId name_id;
  InternedStringId url_id;
};

struct OriginAttributesDictionaryAtoms
{
  InternedStringId appId_id;
  InternedStringId inBrowser_id;
};

struct PageTransitionEventInitAtoms
{
  InternedStringId persisted_id;
};

struct Pbkdf2ParamsAtoms
{
  InternedStringId hash_id;
  InternedStringId iterations_id;
  InternedStringId salt_id;
};

struct PermissionSettingsAtoms
{
  InternedStringId get_id;
  InternedStringId set_id;
  InternedStringId isExplicit_id;
  InternedStringId remove_id;
};

struct PhoneNumberServiceAtoms
{
  InternedStringId fuzzyMatch_id;
  InternedStringId normalize_id;
};

struct PluginCrashedEventInitAtoms
{
  InternedStringId browserDumpID_id;
  InternedStringId gmpPlugin_id;
  InternedStringId pluginDumpID_id;
  InternedStringId pluginFilename_id;
  InternedStringId pluginID_id;
  InternedStringId pluginName_id;
  InternedStringId submittedCrashReport_id;
};

struct PointerEventInitAtoms
{
  InternedStringId height_id;
  InternedStringId isPrimary_id;
  InternedStringId pointerId_id;
  InternedStringId pointerType_id;
  InternedStringId pressure_id;
  InternedStringId tiltX_id;
  InternedStringId tiltY_id;
  InternedStringId width_id;
};

struct PopStateEventInitAtoms
{
  InternedStringId state_id;
};

struct PopupBlockedEventInitAtoms
{
  InternedStringId popupWindowFeatures_id;
  InternedStringId popupWindowName_id;
  InternedStringId popupWindowURI_id;
  InternedStringId requestingWindow_id;
};

struct PositionOptionsAtoms
{
  InternedStringId enableHighAccuracy_id;
  InternedStringId maximumAge_id;
  InternedStringId timeout_id;
};

struct PowerStatsDataAtoms
{
  InternedStringId consumedPower_id;
  InternedStringId timestamp_id;
};

struct PresentationDeviceInfoAtoms
{
  InternedStringId id_id;
  InternedStringId name_id;
  InternedStringId type_id;
};

struct PresentationDeviceInfoManagerAtoms
{
  InternedStringId ondevicechange_id;
  InternedStringId getAll_id;
  InternedStringId forceDiscovery_id;
};

struct ProfileTimelineLayerRectAtoms
{
  InternedStringId height_id;
  InternedStringId width_id;
  InternedStringId x_id;
  InternedStringId y_id;
};

struct ProfileTimelineMarkerAtoms
{
  InternedStringId causeName_id;
  InternedStringId end_id;
  InternedStringId endStack_id;
  InternedStringId eventPhase_id;
  InternedStringId name_id;
  InternedStringId rectangles_id;
  InternedStringId restyleHint_id;
  InternedStringId stack_id;
  InternedStringId start_id;
  InternedStringId type_id;
};

struct ProfileTimelineStackFrameAtoms
{
  InternedStringId asyncCause_id;
  InternedStringId asyncParent_id;
  InternedStringId column_id;
  InternedStringId functionDisplayName_id;
  InternedStringId line_id;
  InternedStringId parent_id;
  InternedStringId source_id;
};

struct ProgressEventInitAtoms
{
  InternedStringId lengthComputable_id;
  InternedStringId loaded_id;
  InternedStringId total_id;
};

struct PromiseDebuggingStateHolderAtoms
{
  InternedStringId reason_id;
  InternedStringId state_id;
  InternedStringId value_id;
};

struct PushEventInitAtoms
{
  InternedStringId data_id;
};

struct PushManagerAtoms
{
  InternedStringId subscribe_id;
  InternedStringId getSubscription_id;
  InternedStringId hasPermission_id;
  InternedStringId setScope_id;
};

struct PushSubscriptionAtoms
{
  InternedStringId endpoint_id;
  InternedStringId unsubscribe_id;
  InternedStringId __jsonifier_id;
  InternedStringId __init_id;
};

struct RTCCodecStatsAtoms
{
  InternedStringId channels_id;
  InternedStringId clockRate_id;
  InternedStringId codec_id;
  InternedStringId parameters_id;
  InternedStringId payloadType_id;
};

struct RTCConfigurationAtoms
{
  InternedStringId iceServers_id;
  InternedStringId peerIdentity_id;
};

struct RTCDataChannelEventInitAtoms
{
  InternedStringId channel_id;
};

struct RTCDataChannelInitAtoms
{
  InternedStringId id_id;
  InternedStringId maxRetransmitNum_id;
  InternedStringId maxRetransmitTime_id;
  InternedStringId maxRetransmits_id;
  InternedStringId negotiated_id;
  InternedStringId ordered_id;
  InternedStringId outOfOrderAllowed_id;
  InternedStringId preset_id;
  InternedStringId protocol_id;
  InternedStringId stream_id;
};

struct RTCIceCandidateInitAtoms
{
  InternedStringId candidate_id;
  InternedStringId sdpMLineIndex_id;
  InternedStringId sdpMid_id;
};

struct RTCIceCandidatePairStatsAtoms
{
  InternedStringId componentId_id;
  InternedStringId localCandidateId_id;
  InternedStringId mozPriority_id;
  InternedStringId nominated_id;
  InternedStringId readable_id;
  InternedStringId remoteCandidateId_id;
  InternedStringId selected_id;
  InternedStringId state_id;
};

struct RTCIceCandidateStatsAtoms
{
  InternedStringId candidateId_id;
  InternedStringId candidateType_id;
  InternedStringId componentId_id;
  InternedStringId ipAddress_id;
  InternedStringId mozLocalTransport_id;
  InternedStringId portNumber_id;
  InternedStringId transport_id;
};

struct RTCIceComponentStatsAtoms
{
  InternedStringId activeConnection_id;
  InternedStringId bytesReceived_id;
  InternedStringId bytesSent_id;
  InternedStringId component_id;
  InternedStringId transportId_id;
};

struct RTCIceServerAtoms
{
  InternedStringId credential_id;
  InternedStringId url_id;
  InternedStringId urls_id;
  InternedStringId username_id;
};

struct RTCIdentityAssertionAtoms
{
  InternedStringId idp_id;
  InternedStringId name_id;
};

struct RTCIdentityAssertionResultAtoms
{
  InternedStringId assertion_id;
  InternedStringId idp_id;
};

struct RTCIdentityProviderAtoms
{
  InternedStringId generateAssertion_id;
  InternedStringId validateAssertion_id;
};

struct RTCIdentityProviderDetailsAtoms
{
  InternedStringId domain_id;
  InternedStringId protocol_id;
};

struct RTCIdentityValidationResultAtoms
{
  InternedStringId contents_id;
  InternedStringId identity_id;
};

struct RTCInboundRTPStreamStatsAtoms
{
  InternedStringId bytesReceived_id;
  InternedStringId discardedPackets_id;
  InternedStringId jitter_id;
  InternedStringId mozAvSyncDelay_id;
  InternedStringId mozJitterBufferDelay_id;
  InternedStringId mozRtt_id;
  InternedStringId packetsLost_id;
  InternedStringId packetsReceived_id;
};

struct RTCMediaStreamStatsAtoms
{
  InternedStringId streamIdentifier_id;
  InternedStringId trackIds_id;
};

struct RTCMediaStreamTrackStatsAtoms
{
  InternedStringId audioLevel_id;
  InternedStringId echoReturnLoss_id;
  InternedStringId echoReturnLossEnhancement_id;
  InternedStringId frameHeight_id;
  InternedStringId frameWidth_id;
  InternedStringId framesCorrupted_id;
  InternedStringId framesDecoded_id;
  InternedStringId framesDropped_id;
  InternedStringId framesPerSecond_id;
  InternedStringId framesReceived_id;
  InternedStringId framesSent_id;
  InternedStringId remoteSource_id;
  InternedStringId ssrcIds_id;
  InternedStringId trackIdentifier_id;
};

struct RTCOfferOptionsAtoms
{
  InternedStringId mandatory_id;
  InternedStringId mozBundleOnly_id;
  InternedStringId mozDontOfferDataChannel_id;
  InternedStringId offerToReceiveAudio_id;
  InternedStringId offerToReceiveVideo_id;
  InternedStringId optional_id;
};

struct RTCOutboundRTPStreamStatsAtoms
{
  InternedStringId bytesSent_id;
  InternedStringId droppedFrames_id;
  InternedStringId packetsSent_id;
  InternedStringId targetBitrate_id;
};

struct RTCPeerConnectionIceEventInitAtoms
{
  InternedStringId candidate_id;
};

struct RTCRTPStreamStatsAtoms
{
  InternedStringId bitrateMean_id;
  InternedStringId bitrateStdDev_id;
  InternedStringId codecId_id;
  InternedStringId framerateMean_id;
  InternedStringId framerateStdDev_id;
  InternedStringId isRemote_id;
  InternedStringId mediaTrackId_id;
  InternedStringId mediaType_id;
  InternedStringId remoteId_id;
  InternedStringId ssrc_id;
  InternedStringId transportId_id;
};

struct RTCRtpReceiverAtoms
{
  InternedStringId track_id;
};

struct RTCRtpSenderAtoms
{
  InternedStringId track_id;
  InternedStringId replaceTrack_id;
};

struct RTCSessionDescriptionInitAtoms
{
  InternedStringId sdp_id;
  InternedStringId type_id;
};

struct RTCStatsAtoms
{
  InternedStringId id_id;
  InternedStringId timestamp_id;
  InternedStringId type_id;
};

struct RTCStatsReportAtoms
{
  InternedStringId mozPcid_id;
  InternedStringId forEach_id;
  InternedStringId get_id;
  InternedStringId has_id;
};

struct RTCStatsReportInternalAtoms
{
  InternedStringId closed_id;
  InternedStringId codecStats_id;
  InternedStringId iceCandidatePairStats_id;
  InternedStringId iceCandidateStats_id;
  InternedStringId iceComponentStats_id;
  InternedStringId inboundRTPStreamStats_id;
  InternedStringId localSdp_id;
  InternedStringId mediaStreamStats_id;
  InternedStringId mediaStreamTrackStats_id;
  InternedStringId outboundRTPStreamStats_id;
  InternedStringId pcid_id;
  InternedStringId remoteSdp_id;
  InternedStringId timestamp_id;
  InternedStringId transportStats_id;
};

struct RTCTransportStatsAtoms
{
  InternedStringId bytesReceived_id;
  InternedStringId bytesSent_id;
};

struct RecordErrorEventInitAtoms
{
  InternedStringId name_id;
};

struct RegistrationOptionsAtoms
{
  InternedStringId scope_id;
};

struct RequestFullscreenOptionsAtoms
{
  InternedStringId vrDisplay_id;
};

struct RequestInitAtoms
{
  InternedStringId body_id;
  InternedStringId cache_id;
  InternedStringId credentials_id;
  InternedStringId headers_id;
  InternedStringId method_id;
  InternedStringId mode_id;
};

struct RequestMediaKeySystemAccessNotificationAtoms
{
  InternedStringId keySystem_id;
  InternedStringId status_id;
};

struct RequestSyncAppAtoms
{
  InternedStringId origin_id;
  InternedStringId manifestURL_id;
  InternedStringId isInBrowserElement_id;
};

struct RequestSyncManagerAtoms
{
  InternedStringId registrations_id;
};

struct RequestSyncSchedulerAtoms
{
  InternedStringId register_id;
  InternedStringId unregister_id;
  InternedStringId registrations_id;
  InternedStringId registration_id;
};

struct RequestSyncTaskAtoms
{
  InternedStringId app_id;
  InternedStringId state_id;
  InternedStringId overwrittenMinInterval_id;
  InternedStringId task_id;
  InternedStringId lastSync_id;
  InternedStringId wakeUpPage_id;
  InternedStringId oneShot_id;
  InternedStringId minInterval_id;
  InternedStringId wifiOnly_id;
  InternedStringId data_id;
  InternedStringId setPolicy_id;
  InternedStringId runNow_id;
};

struct RequestTaskFullAtoms
{
  InternedStringId lastSync_id;
  InternedStringId task_id;
};

struct RequestTaskParamsAtoms
{
  InternedStringId data_id;
  InternedStringId minInterval_id;
  InternedStringId oneShot_id;
  InternedStringId wakeUpPage_id;
  InternedStringId wifiOnly_id;
};

struct ResourceStatsAlarmAtoms
{
  InternedStringId alarmId_id;
  InternedStringId type_id;
  InternedStringId component_id;
  InternedStringId serviceType_id;
  InternedStringId manifestURL_id;
  InternedStringId threshold_id;
  InternedStringId data_id;
};

struct ResourceStatsAlarmOptionsAtoms
{
  InternedStringId data_id;
  InternedStringId startTime_id;
};

struct ResourceStatsAtoms
{
  InternedStringId type_id;
  InternedStringId component_id;
  InternedStringId serviceType_id;
  InternedStringId manifestURL_id;
  InternedStringId getData_id;
  InternedStringId start_id;
  InternedStringId end_id;
};

struct ResourceStatsManagerAtoms
{
  InternedStringId getStats_id;
  InternedStringId clearStats_id;
  InternedStringId clearAllStats_id;
  InternedStringId addAlarm_id;
  InternedStringId getAlarms_id;
  InternedStringId removeAlarm_id;
  InternedStringId removeAllAlarms_id;
  InternedStringId getAvailableComponents_id;
  InternedStringId resourceTypes_id;
  InternedStringId sampleRate_id;
  InternedStringId maxStorageAge_id;
  InternedStringId __init_id;
};

struct ResourceStatsOptionsAtoms
{
  InternedStringId component_id;
  InternedStringId manifestURL_id;
  InternedStringId serviceType_id;
};

struct ResponseInitAtoms
{
  InternedStringId headers_id;
  InternedStringId status_id;
  InternedStringId statusText_id;
};

struct RsaHashedImportParamsAtoms
{
  InternedStringId hash_id;
};

struct RsaHashedKeyAlgorithmAtoms
{
  InternedStringId hash_id;
  InternedStringId modulusLength_id;
  InternedStringId publicExponent_id;
};

struct RsaHashedKeyGenParamsAtoms
{
  InternedStringId hash_id;
  InternedStringId modulusLength_id;
  InternedStringId publicExponent_id;
};

struct RsaOaepParamsAtoms
{
  InternedStringId label_id;
};

struct RsaOtherPrimesInfoAtoms
{
  InternedStringId d_id;
  InternedStringId r_id;
  InternedStringId t_id;
};

struct SEChannelAtoms
{
  InternedStringId session_id;
  InternedStringId openResponse_id;
  InternedStringId isClosed_id;
  InternedStringId type_id;
  InternedStringId transmit_id;
  InternedStringId close_id;
};

struct SECommandAtoms
{
  InternedStringId cla_id;
  InternedStringId data_id;
  InternedStringId ins_id;
  InternedStringId le_id;
  InternedStringId p1_id;
  InternedStringId p2_id;
};

struct SEManagerAtoms
{
  InternedStringId getSEReaders_id;
};

struct SEReaderAtoms
{
  InternedStringId isSEPresent_id;
  InternedStringId type_id;
  InternedStringId openSession_id;
  InternedStringId closeAll_id;
};

struct SEResponseAtoms
{
  InternedStringId channel_id;
  InternedStringId sw1_id;
  InternedStringId sw2_id;
  InternedStringId data_id;
};

struct SESessionAtoms
{
  InternedStringId reader_id;
  InternedStringId isClosed_id;
  InternedStringId openLogicalChannel_id;
  InternedStringId closeAll_id;
};

struct SVGBoundingBoxOptionsAtoms
{
  InternedStringId clipped_id;
  InternedStringId fill_id;
  InternedStringId markers_id;
  InternedStringId stroke_id;
};

struct ScrollFrameDataAtoms
{
  InternedStringId entries_id;
  InternedStringId scrollId_id;
};

struct ScrollFrameDataEntryAtoms
{
  InternedStringId key_id;
  InternedStringId value_id;
};

struct ScrollIntoViewOptionsAtoms
{
  InternedStringId block_id;
};

struct ScrollOptionsAtoms
{
  InternedStringId behavior_id;
};

struct ScrollToOptionsAtoms
{
  InternedStringId left_id;
  InternedStringId top_id;
};

struct ScrollViewChangeEventInitAtoms
{
  InternedStringId state_id;
};

struct SelectionStateChangedEventInitAtoms
{
  InternedStringId boundingClientRect_id;
  InternedStringId selectedText_id;
  InternedStringId states_id;
  InternedStringId visible_id;
};

struct SettingChangeAtoms
{
  InternedStringId settingName_id;
  InternedStringId settingValue_id;
};

struct SettingChangeNotificationAtoms
{
  InternedStringId isInternalChange_id;
  InternedStringId key_id;
  InternedStringId value_id;
};

struct SettingsLockAtoms
{
  InternedStringId closed_id;
  InternedStringId set_id;
  InternedStringId get_id;
  InternedStringId clear_id;
  InternedStringId onsettingstransactionsuccess_id;
  InternedStringId onsettingstransactionfailure_id;
};

struct SettingsManagerAtoms
{
  InternedStringId createLock_id;
  InternedStringId addObserver_id;
  InternedStringId removeObserver_id;
  InternedStringId onsettingchange_id;
};

struct SmsSegmentInfoAtoms
{
  InternedStringId charsAvailableInLastSegment_id;
  InternedStringId charsPerSegment_id;
  InternedStringId segments_id;
};

struct SmsSendParametersAtoms
{
  InternedStringId serviceId_id;
};

struct SmscAddressAtoms
{
  InternedStringId address_id;
  InternedStringId typeOfAddress_id;
};

struct SocketElementAtoms
{
  InternedStringId active_id;
  InternedStringId host_id;
  InternedStringId port_id;
  InternedStringId received_id;
  InternedStringId sent_id;
  InternedStringId tcp_id;
};

struct SocketsDictAtoms
{
  InternedStringId received_id;
  InternedStringId sent_id;
  InternedStringId sockets_id;
};

struct SpeechRecognitionErrorInitAtoms
{
  InternedStringId error_id;
  InternedStringId message_id;
};

struct SpeechRecognitionEventInitAtoms
{
  InternedStringId emma_id;
  InternedStringId interpretation_id;
  InternedStringId resultIndex_id;
  InternedStringId results_id;
};

struct SpeechSynthesisEventInitAtoms
{
  InternedStringId charIndex_id;
  InternedStringId elapsedTime_id;
  InternedStringId name_id;
};

struct StorageEventInitAtoms
{
  InternedStringId key_id;
  InternedStringId newValue_id;
  InternedStringId oldValue_id;
  InternedStringId storageArea_id;
  InternedStringId url_id;
};

struct StyleRuleChangeEventInitAtoms
{
  InternedStringId rule_id;
  InternedStringId stylesheet_id;
};

struct StyleSheetApplicableStateChangeEventInitAtoms
{
  InternedStringId applicable_id;
  InternedStringId stylesheet_id;
};

struct StyleSheetChangeEventInitAtoms
{
  InternedStringId documentSheet_id;
  InternedStringId stylesheet_id;
};

struct TVCurrentChannelChangedEventInitAtoms
{
  InternedStringId channel_id;
};

struct TVCurrentSourceChangedEventInitAtoms
{
  InternedStringId source_id;
};

struct TVEITBroadcastedEventInitAtoms
{
  InternedStringId programs_id;
};

struct TVGetProgramsOptionsAtoms
{
  InternedStringId duration_id;
  InternedStringId startTime_id;
};

struct TVScanningStateChangedEventInitAtoms
{
  InternedStringId channel_id;
  InternedStringId state_id;
};

struct TVStartScanningOptionsAtoms
{
  InternedStringId isRescanned_id;
};

struct TetheringConfigurationAtoms
{
  InternedStringId dns1_id;
  InternedStringId dns2_id;
  InternedStringId endIp_id;
  InternedStringId ip_id;
  InternedStringId prefix_id;
  InternedStringId startIp_id;
  InternedStringId wifiConfig_id;
};

struct TextDecodeOptionsAtoms
{
  InternedStringId stream_id;
};

struct TextDecoderOptionsAtoms
{
  InternedStringId fatal_id;
};

struct TrackEventInitAtoms
{
  InternedStringId track_id;
};

struct TransitionEventInitAtoms
{
  InternedStringId elapsedTime_id;
  InternedStringId propertyName_id;
  InternedStringId pseudoElement_id;
};

struct TreeCellInfoAtoms
{
  InternedStringId childElt_id;
  InternedStringId col_id;
  InternedStringId row_id;
};

struct TypeOfAddressAtoms
{
  InternedStringId numberPlanIdentification_id;
  InternedStringId typeOfNumber_id;
};

struct UDPMessageEventInitAtoms
{
  InternedStringId data_id;
  InternedStringId remoteAddress_id;
  InternedStringId remotePort_id;
};

struct UDPOptionsAtoms
{
  InternedStringId addressReuse_id;
  InternedStringId localAddress_id;
  InternedStringId localPort_id;
  InternedStringId loopback_id;
  InternedStringId remoteAddress_id;
  InternedStringId remotePort_id;
};

struct UIEventInitAtoms
{
  InternedStringId detail_id;
  InternedStringId view_id;
};

struct USSDReceivedEventInitAtoms
{
  InternedStringId message_id;
  InternedStringId serviceId_id;
  InternedStringId session_id;
};

struct UncaughtRejectionObserverAtoms
{
  InternedStringId onLeftUncaught_id;
  InternedStringId onConsumed_id;
};

struct UserProximityEventInitAtoms
{
  InternedStringId near_id;
};

struct VRFieldOfViewInitAtoms
{
  InternedStringId downDegrees_id;
  InternedStringId leftDegrees_id;
  InternedStringId rightDegrees_id;
  InternedStringId upDegrees_id;
};

struct WebGLContextAttributesAtoms
{
  InternedStringId alpha_id;
  InternedStringId antialias_id;
  InternedStringId depth_id;
  InternedStringId failIfMajorPerformanceCaveat_id;
  InternedStringId premultipliedAlpha_id;
  InternedStringId preserveDrawingBuffer_id;
  InternedStringId stencil_id;
};

struct WebSocketDictAtoms
{
  InternedStringId websockets_id;
};

struct WebSocketElementAtoms
{
  InternedStringId encrypted_id;
  InternedStringId hostport_id;
  InternedStringId msgreceived_id;
  InternedStringId msgsent_id;
  InternedStringId receivedsize_id;
  InternedStringId sentsize_id;
};

struct WheelEventInitAtoms
{
  InternedStringId deltaMode_id;
  InternedStringId deltaX_id;
  InternedStringId deltaY_id;
  InternedStringId deltaZ_id;
};

struct WifiCertServiceResultOptionsAtoms
{
  InternedStringId id_id;
  InternedStringId nickname_id;
  InternedStringId status_id;
  InternedStringId usageFlag_id;
};

struct WifiCommandOptionsAtoms
{
  InternedStringId cmd_id;
  InternedStringId id_id;
  InternedStringId request_id;
};

struct WifiResultOptionsAtoms
{
  InternedStringId id_id;
  InternedStringId reply_id;
  InternedStringId status_id;
};

struct WifiTetheringConfigAtoms
{
  InternedStringId key_id;
  InternedStringId security_id;
  InternedStringId ssid_id;
};

struct XPathNSResolverAtoms
{
  InternedStringId lookupNamespaceURI_id;
};

struct mozContactAtoms
{
  InternedStringId id_id;
  InternedStringId published_id;
  InternedStringId updated_id;
  InternedStringId bday_id;
  InternedStringId anniversary_id;
  InternedStringId sex_id;
  InternedStringId genderIdentity_id;
  InternedStringId photo_id;
  InternedStringId adr_id;
  InternedStringId email_id;
  InternedStringId url_id;
  InternedStringId impp_id;
  InternedStringId tel_id;
  InternedStringId name_id;
  InternedStringId honorificPrefix_id;
  InternedStringId givenName_id;
  InternedStringId phoneticGivenName_id;
  InternedStringId additionalName_id;
  InternedStringId familyName_id;
  InternedStringId phoneticFamilyName_id;
  InternedStringId honorificSuffix_id;
  InternedStringId nickname_id;
  InternedStringId category_id;
  InternedStringId org_id;
  InternedStringId jobTitle_id;
  InternedStringId note_id;
  InternedStringId key_id;
  InternedStringId init_id;
  InternedStringId setMetadata_id;
  InternedStringId __jsonifier_id;
  InternedStringId __init_id;
};

struct mozRTCIceCandidateAtoms
{
  InternedStringId candidate_id;
  InternedStringId sdpMid_id;
  InternedStringId sdpMLineIndex_id;
  InternedStringId __jsonifier_id;
  InternedStringId __init_id;
};

struct mozRTCPeerConnectionAtoms
{
  InternedStringId setIdentityProvider_id;
  InternedStringId getIdentityAssertion_id;
  InternedStringId createOffer_id;
  InternedStringId createAnswer_id;
  InternedStringId setLocalDescription_id;
  InternedStringId setRemoteDescription_id;
  InternedStringId localDescription_id;
  InternedStringId remoteDescription_id;
  InternedStringId signalingState_id;
  InternedStringId updateIce_id;
  InternedStringId addIceCandidate_id;
  InternedStringId iceGatheringState_id;
  InternedStringId iceConnectionState_id;
  InternedStringId peerIdentity_id;
  InternedStringId idpLoginUrl_id;
  InternedStringId id_id;
  InternedStringId getConfiguration_id;
  InternedStringId getLocalStreams_id;
  InternedStringId getRemoteStreams_id;
  InternedStringId getStreamById_id;
  InternedStringId addStream_id;
  InternedStringId removeStream_id;
  InternedStringId addTrack_id;
  InternedStringId removeTrack_id;
  InternedStringId getSenders_id;
  InternedStringId getReceivers_id;
  InternedStringId close_id;
  InternedStringId onnegotiationneeded_id;
  InternedStringId onicecandidate_id;
  InternedStringId onsignalingstatechange_id;
  InternedStringId onaddstream_id;
  InternedStringId onaddtrack_id;
  InternedStringId onremovestream_id;
  InternedStringId oniceconnectionstatechange_id;
  InternedStringId getStats_id;
  InternedStringId createDataChannel_id;
  InternedStringId ondatachannel_id;
  InternedStringId __init_id;
};

struct mozRTCPeerConnectionStaticAtoms
{
  InternedStringId registerPeerConnectionLifecycleCallback_id;
  InternedStringId __init_id;
};

struct mozRTCSessionDescriptionAtoms
{
  InternedStringId type_id;
  InternedStringId sdp_id;
  InternedStringId __jsonifier_id;
  InternedStringId __init_id;
};

struct PerThreadAtomCache : public APZBucketAtoms,
                            public APZTestDataAtoms,
                            public ActivityOptionsAtoms,
                            public ActivityRequestHandlerAtoms,
                            public AdoptDownloadDictAtoms,
                            public AesCbcParamsAtoms,
                            public AesCtrParamsAtoms,
                            public AesDerivedKeyParamsAtoms,
                            public AesGcmParamsAtoms,
                            public AesKeyAlgorithmAtoms,
                            public AesKeyGenParamsAtoms,
                            public AlarmsManagerAtoms,
                            public AlgorithmAtoms,
                            public AnimationEventInitAtoms,
                            public AppInfoAtoms,
                            public AppNotificationServiceOptionsAtoms,
                            public ArchiveReaderOptionsAtoms,
                            public AsyncScrollEventDetailAtoms,
                            public AutocompleteErrorEventInitAtoms,
                            public AutocompleteInfoAtoms,
                            public BeforeAfterKeyboardEventInitAtoms,
                            public BlobEventInitAtoms,
                            public BlobPropertyBagAtoms,
                            public BoxQuadOptionsAtoms,
                            public BrowserElementDownloadOptionsAtoms,
                            public BrowserFeedWriterAtoms,
                            public CFStateChangeEventInitAtoms,
                            public CSPAtoms,
                            public CSPPoliciesAtoms,
                            public CSPReportAtoms,
                            public CSPReportPropertiesAtoms,
                            public CSSTokenAtoms,
                            public CacheBatchOperationAtoms,
                            public CacheQueryOptionsAtoms,
                            public CallEventInitAtoms,
                            public CallGroupErrorEventInitAtoms,
                            public CameraClosedEventInitAtoms,
                            public CameraConfigurationAtoms,
                            public CameraConfigurationEventInitAtoms,
                            public CameraDetectedFaceInitAtoms,
                            public CameraFacesDetectedEventInitAtoms,
                            public CameraGetPromiseDataAtoms,
                            public CameraPictureOptionsAtoms,
                            public CameraPositionAtoms,
                            public CameraRegionAtoms,
                            public CameraSizeAtoms,
                            public CameraStartRecordingOptionsAtoms,
                            public CameraStateChangeEventInitAtoms,
                            public CaretStateChangedEventInitAtoms,
                            public ChromeFilePropertyBagAtoms,
                            public ChromeNotificationsAtoms,
                            public ClientQueryOptionsAtoms,
                            public ClipboardEventInitAtoms,
                            public CloseEventInitAtoms,
                            public CompositionClauseParametersAtoms,
                            public ConnStatusDictAtoms,
                            public ConsoleCounterAtoms,
                            public ConsoleCounterErrorAtoms,
                            public ConsoleEventAtoms,
                            public ConsoleProfileEventAtoms,
                            public ConsoleStackEntryAtoms,
                            public ConsoleTimerEndAtoms,
                            public ConsoleTimerErrorAtoms,
                            public ConsoleTimerStartAtoms,
                            public ConstrainBooleanParametersAtoms,
                            public ConstrainDOMStringParametersAtoms,
                            public ConstrainDoubleRangeAtoms,
                            public ConstrainLongRangeAtoms,
                            public ContactAddressAtoms,
                            public ContactFieldAtoms,
                            public ContactFindOptionsAtoms,
                            public ContactFindSortOptionsAtoms,
                            public ContactManagerAtoms,
                            public ContactPropertiesAtoms,
                            public ContactTelFieldAtoms,
                            public ContextAttributes2DAtoms,
                            public ConvertCoordinateOptionsAtoms,
                            public CreateFileOptionsAtoms,
                            public CryptoKeyPairAtoms,
                            public CustomEventInitAtoms,
                            public DNSCacheDictAtoms,
                            public DNSLookupDictAtoms,
                            public DOMApplicationAtoms,
                            public DOMApplicationsManagerAtoms,
                            public DOMApplicationsRegistryAtoms,
                            public DOMDownloadAtoms,
                            public DOMDownloadManagerAtoms,
                            public DOMPointInitAtoms,
                            public DOMTransactionAtoms,
                            public DOMTransactionEventInitAtoms,
                            public DOMWindowResizeEventDetailAtoms,
                            public DataErrorEventInitAtoms,
                            public DataStoreChangeEventInitAtoms,
                            public DataStoreCursorImplAtoms,
                            public DataStoreImplAtoms,
                            public DataStoreRevisionDataAtoms,
                            public DataStoreTaskAtoms,
                            public DeprecatedRTCOfferOptionsSetAtoms,
                            public DeviceAccelerationInitAtoms,
                            public DeviceLightEventInitAtoms,
                            public DeviceMotionEventInitAtoms,
                            public DeviceOrientationEventInitAtoms,
                            public DeviceProximityEventInitAtoms,
                            public DeviceRotationRateInitAtoms,
                            public DeviceStorageAreaChangedEventInitAtoms,
                            public DeviceStorageChangeEventInitAtoms,
                            public DeviceStorageEnumerationParametersAtoms,
                            public DhImportKeyParamsAtoms,
                            public DhKeyAlgorithmAtoms,
                            public DhKeyDeriveParamsAtoms,
                            public DhKeyGenParamsAtoms,
                            public DnsCacheEntryAtoms,
                            public DownloadEventInitAtoms,
                            public DragEventInitAtoms,
                            public EcKeyAlgorithmAtoms,
                            public EcKeyGenParamsAtoms,
                            public EcKeyImportParamsAtoms,
                            public EcdhKeyDeriveParamsAtoms,
                            public EcdsaParamsAtoms,
                            public ElementRegistrationOptionsAtoms,
                            public EngineeringModeAtoms,
                            public ErrorEventInitAtoms,
                            public EventInitAtoms,
                            public EventListenerAtoms,
                            public EventModifierInitAtoms,
                            public EventSourceInitAtoms,
                            public ExternalAtoms,
                            public FetchEventInitAtoms,
                            public FilePropertyBagAtoms,
                            public FocusEventInitAtoms,
                            public FontFaceDescriptorsAtoms,
                            public FontFaceSetIteratorResultAtoms,
                            public FontFaceSetLoadEventInitAtoms,
                            public FrameUniformityAtoms,
                            public FrameUniformityResultsAtoms,
                            public GamepadAxisMoveEventInitAtoms,
                            public GamepadButtonEventInitAtoms,
                            public GamepadEventInitAtoms,
                            public GetNotificationOptionsAtoms,
                            public HalfOpenInfoDictAtoms,
                            public HashChangeEventInitAtoms,
                            public HeapSnapshotBoundariesAtoms,
                            public HitRegionOptionsAtoms,
                            public HmacDerivedKeyParamsAtoms,
                            public HmacImportParamsAtoms,
                            public HmacKeyAlgorithmAtoms,
                            public HmacKeyGenParamsAtoms,
                            public HttpConnDictAtoms,
                            public HttpConnInfoAtoms,
                            public HttpConnectionElementAtoms,
                            public IDBFileMetadataParametersAtoms,
                            public IDBIndexParametersAtoms,
                            public IDBObjectStoreParametersAtoms,
                            public IDBOpenDBOptionsAtoms,
                            public IDBVersionChangeEventInitAtoms,
                            public IccCardLockRetryCountAtoms,
                            public IccCardLockStatusAtoms,
                            public IccChangeEventInitAtoms,
                            public IccSetCardLockOptionsAtoms,
                            public IccUnlockCardLockOptionsAtoms,
                            public ImageCaptureErrorEventInitAtoms,
                            public InputEventInitAtoms,
                            public InspectorRGBATupleAtoms,
                            public InspectorRGBTripleAtoms,
                            public InstallParametersAtoms,
                            public InstallTriggerDataAtoms,
                            public InstallTriggerImplAtoms,
                            public JsonWebKeyAtoms,
                            public KeyAlgorithmAtoms,
                            public KeyboardEventInitAtoms,
                            public LanguageDescAtoms,
                            public LifecycleCallbacksAtoms,
                            public MediaKeyMessageEventInitAtoms,
                            public MediaKeyNeededEventInitAtoms,
                            public MediaKeySystemOptionsAtoms,
                            public MediaRecorderOptionsAtoms,
                            public MediaStreamConstraintsAtoms,
                            public MediaStreamEventInitAtoms,
                            public MediaStreamTrackEventInitAtoms,
                            public MediaTrackConstraintSetAtoms,
                            public MediaTrackConstraintsAtoms,
                            public MessageEventInitAtoms,
                            public MmsAttachmentAtoms,
                            public MmsDeliveryInfoAtoms,
                            public MmsParametersAtoms,
                            public MmsSendParametersAtoms,
                            public MobileMessageFilterAtoms,
                            public MouseEventInitAtoms,
                            public MozApplicationEventInitAtoms,
                            public MozCallBarringOptionsAtoms,
                            public MozCallForwardingOptionsAtoms,
                            public MozCellBroadcastEventInitAtoms,
                            public MozClirModeEventInitAtoms,
                            public MozClirStatusAtoms,
                            public MozContactChangeEventInitAtoms,
                            public MozEmergencyCbModeEventInitAtoms,
                            public MozIdleObserverAtoms,
                            public MozInputContextAtoms,
                            public MozInputMethodAtoms,
                            public MozInputMethodManagerAtoms,
                            public MozInterAppConnectionAtoms,
                            public MozInterAppConnectionRequestAtoms,
                            public MozInterAppMessageEventInitAtoms,
                            public MozInterAppMessagePortAtoms,
                            public MozMMIResultAtoms,
                            public MozMessageDeletedEventInitAtoms,
                            public MozMmsEventInitAtoms,
                            public MozOtaStatusEventInitAtoms,
                            public MozPluginParameterAtoms,
                            public MozSelfSupportAtoms,
                            public MozSettingsEventInitAtoms,
                            public MozSettingsTransactionEventInitAtoms,
                            public MozSmsEventInitAtoms,
                            public MozStkBipMessageAtoms,
                            public MozStkBrowserSettingAtoms,
                            public MozStkBrowserTerminationEventAtoms,
                            public MozStkCallEventAtoms,
                            public MozStkCommandAtoms,
                            public MozStkCommandEventInitAtoms,
                            public MozStkDurationAtoms,
                            public MozStkGeneralEventAtoms,
                            public MozStkIconAtoms,
                            public MozStkIconContainerAtoms,
                            public MozStkInputAtoms,
                            public MozStkItemAtoms,
                            public MozStkLanguageSelectionEventAtoms,
                            public MozStkLocalInfoAtoms,
                            public MozStkLocationEventAtoms,
                            public MozStkLocationInfoAtoms,
                            public MozStkMenuAtoms,
                            public MozStkPlayToneAtoms,
                            public MozStkProvideLocalInfoAtoms,
                            public MozStkResponseAtoms,
                            public MozStkSetUpCallAtoms,
                            public MozStkSetUpEventListAtoms,
                            public MozStkTextMessageAtoms,
                            public MozStkTimerAtoms,
                            public MozTetheringManagerAtoms,
                            public MozVoicemailEventInitAtoms,
                            public MozXMLHttpRequestParametersAtoms,
                            public MutationObserverInitAtoms,
                            public MutationObservingInfoAtoms,
                            public NativeOSFileReadOptionsAtoms,
                            public NetworkCommandOptionsAtoms,
                            public NetworkResultOptionsAtoms,
                            public NetworkStatsDataAtoms,
                            public NodeFilterAtoms,
                            public NotificationBehaviorAtoms,
                            public NotificationOptionsAtoms,
                            public OpenWindowEventDetailAtoms,
                            public OriginAttributesDictionaryAtoms,
                            public PageTransitionEventInitAtoms,
                            public Pbkdf2ParamsAtoms,
                            public PermissionSettingsAtoms,
                            public PhoneNumberServiceAtoms,
                            public PluginCrashedEventInitAtoms,
                            public PointerEventInitAtoms,
                            public PopStateEventInitAtoms,
                            public PopupBlockedEventInitAtoms,
                            public PositionOptionsAtoms,
                            public PowerStatsDataAtoms,
                            public PresentationDeviceInfoAtoms,
                            public PresentationDeviceInfoManagerAtoms,
                            public ProfileTimelineLayerRectAtoms,
                            public ProfileTimelineMarkerAtoms,
                            public ProfileTimelineStackFrameAtoms,
                            public ProgressEventInitAtoms,
                            public PromiseDebuggingStateHolderAtoms,
                            public PushEventInitAtoms,
                            public PushManagerAtoms,
                            public PushSubscriptionAtoms,
                            public RTCCodecStatsAtoms,
                            public RTCConfigurationAtoms,
                            public RTCDataChannelEventInitAtoms,
                            public RTCDataChannelInitAtoms,
                            public RTCIceCandidateInitAtoms,
                            public RTCIceCandidatePairStatsAtoms,
                            public RTCIceCandidateStatsAtoms,
                            public RTCIceComponentStatsAtoms,
                            public RTCIceServerAtoms,
                            public RTCIdentityAssertionAtoms,
                            public RTCIdentityAssertionResultAtoms,
                            public RTCIdentityProviderAtoms,
                            public RTCIdentityProviderDetailsAtoms,
                            public RTCIdentityValidationResultAtoms,
                            public RTCInboundRTPStreamStatsAtoms,
                            public RTCMediaStreamStatsAtoms,
                            public RTCMediaStreamTrackStatsAtoms,
                            public RTCOfferOptionsAtoms,
                            public RTCOutboundRTPStreamStatsAtoms,
                            public RTCPeerConnectionIceEventInitAtoms,
                            public RTCRTPStreamStatsAtoms,
                            public RTCRtpReceiverAtoms,
                            public RTCRtpSenderAtoms,
                            public RTCSessionDescriptionInitAtoms,
                            public RTCStatsAtoms,
                            public RTCStatsReportAtoms,
                            public RTCStatsReportInternalAtoms,
                            public RTCTransportStatsAtoms,
                            public RecordErrorEventInitAtoms,
                            public RegistrationOptionsAtoms,
                            public RequestFullscreenOptionsAtoms,
                            public RequestInitAtoms,
                            public RequestMediaKeySystemAccessNotificationAtoms,
                            public RequestSyncAppAtoms,
                            public RequestSyncManagerAtoms,
                            public RequestSyncSchedulerAtoms,
                            public RequestSyncTaskAtoms,
                            public RequestTaskFullAtoms,
                            public RequestTaskParamsAtoms,
                            public ResourceStatsAlarmAtoms,
                            public ResourceStatsAlarmOptionsAtoms,
                            public ResourceStatsAtoms,
                            public ResourceStatsManagerAtoms,
                            public ResourceStatsOptionsAtoms,
                            public ResponseInitAtoms,
                            public RsaHashedImportParamsAtoms,
                            public RsaHashedKeyAlgorithmAtoms,
                            public RsaHashedKeyGenParamsAtoms,
                            public RsaOaepParamsAtoms,
                            public RsaOtherPrimesInfoAtoms,
                            public SEChannelAtoms,
                            public SECommandAtoms,
                            public SEManagerAtoms,
                            public SEReaderAtoms,
                            public SEResponseAtoms,
                            public SESessionAtoms,
                            public SVGBoundingBoxOptionsAtoms,
                            public ScrollFrameDataAtoms,
                            public ScrollFrameDataEntryAtoms,
                            public ScrollIntoViewOptionsAtoms,
                            public ScrollOptionsAtoms,
                            public ScrollToOptionsAtoms,
                            public ScrollViewChangeEventInitAtoms,
                            public SelectionStateChangedEventInitAtoms,
                            public SettingChangeAtoms,
                            public SettingChangeNotificationAtoms,
                            public SettingsLockAtoms,
                            public SettingsManagerAtoms,
                            public SmsSegmentInfoAtoms,
                            public SmsSendParametersAtoms,
                            public SmscAddressAtoms,
                            public SocketElementAtoms,
                            public SocketsDictAtoms,
                            public SpeechRecognitionErrorInitAtoms,
                            public SpeechRecognitionEventInitAtoms,
                            public SpeechSynthesisEventInitAtoms,
                            public StorageEventInitAtoms,
                            public StyleRuleChangeEventInitAtoms,
                            public StyleSheetApplicableStateChangeEventInitAtoms,
                            public StyleSheetChangeEventInitAtoms,
                            public TVCurrentChannelChangedEventInitAtoms,
                            public TVCurrentSourceChangedEventInitAtoms,
                            public TVEITBroadcastedEventInitAtoms,
                            public TVGetProgramsOptionsAtoms,
                            public TVScanningStateChangedEventInitAtoms,
                            public TVStartScanningOptionsAtoms,
                            public TetheringConfigurationAtoms,
                            public TextDecodeOptionsAtoms,
                            public TextDecoderOptionsAtoms,
                            public TrackEventInitAtoms,
                            public TransitionEventInitAtoms,
                            public TreeCellInfoAtoms,
                            public TypeOfAddressAtoms,
                            public UDPMessageEventInitAtoms,
                            public UDPOptionsAtoms,
                            public UIEventInitAtoms,
                            public USSDReceivedEventInitAtoms,
                            public UncaughtRejectionObserverAtoms,
                            public UserProximityEventInitAtoms,
                            public VRFieldOfViewInitAtoms,
                            public WebGLContextAttributesAtoms,
                            public WebSocketDictAtoms,
                            public WebSocketElementAtoms,
                            public WheelEventInitAtoms,
                            public WifiCertServiceResultOptionsAtoms,
                            public WifiCommandOptionsAtoms,
                            public WifiResultOptionsAtoms,
                            public WifiTetheringConfigAtoms,
                            public XPathNSResolverAtoms,
                            public mozContactAtoms,
                            public mozRTCIceCandidateAtoms,
                            public mozRTCPeerConnectionAtoms,
                            public mozRTCPeerConnectionStaticAtoms,
                            public mozRTCSessionDescriptionAtoms
{
};

} // namespace dom
} // namespace mozilla


#endif // mozilla_dom_GeneratedAtomList_h
