/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/. */

#include "nsISupports.idl"

interface nsIURI;
interface nsIChannel;
interface nsILoadInfo;

[scriptable, uuid(c0c19db9-1b5a-4ac5-b656-ed6f8149fa48)]
interface nsIAboutModule : nsISupports 
{

    /**
     * Constructs a new channel for the about protocol module. 
     *
     * @param aURI the uri of the new channel
     * @param aLoadInfo the loadinfo of the new channel
     */
    nsIChannel newChannel(in nsIURI aURI,
                          in nsILoadInfo aLoadInfo);

    /**
     * A flag that indicates whether a URI is safe for untrusted
     * content.  If it is, web pages and so forth will be allowed to
     * link to this about: URI (unless MAKE_UNLINKABLE is also specified),
     * and the about: protocol handler will enforce that the principal
     * of channels created for it be based on their originalURI or URI
     * (depending on the channel flags), by setting their "owner" to null.
     * Otherwise, only chrome will be able to link to it.
     */
    const unsigned long URI_SAFE_FOR_UNTRUSTED_CONTENT = (1 << 0);

    /**
     * A flag that indicates whether script should be enabled for the
     * given about: URI even if it's disabled in general.
     */
    const unsigned long ALLOW_SCRIPT = (1 << 1);

    /**
     * A flag that indicates whether this about: URI doesn't want to be listed
     * in about:about, especially if it's not useful without a query string.
     */
    const unsigned long HIDE_FROM_ABOUTABOUT = (1 << 2);

    /**
     * A flag that indicates whether this about: URI wants Indexed DB enabled.
     */
    const unsigned long ENABLE_INDEXED_DB = (1 << 3);

    /**
     * A flag that indicates that this URI can be loaded in a child process
     */
    const unsigned long URI_CAN_LOAD_IN_CHILD = (1 << 4);

    /**
     * A flag that indicates that this URI must be loaded in a child process
     */
    const unsigned long URI_MUST_LOAD_IN_CHILD = (1 << 5);

    /**
     * A flag that indicates that this URI should be unlinkable despite being
     * safe for untrusted content.
     */
    const unsigned long MAKE_UNLINKABLE = (1 << 6);

    /**
     * A method to get the flags that apply to a given about: URI.  The URI
     * passed in is guaranteed to be one of the URIs that this module
     * registered to deal with.
     */
    unsigned long getURIFlags(in nsIURI aURI);

    /**
     * Returns the Indexed DB origin's postfix used for the given about: URI.
     * If the postfix returned is null then the URI's path (e.g. "home" for
     * about:home) will be used to construct the origin.
     */
    DOMString getIndexedDBOriginPostfix(in nsIURI aURI);
};

%{C++

#define NS_ABOUT_MODULE_CONTRACTID        "@mozilla.org/network/protocol/about;1" 
#define NS_ABOUT_MODULE_CONTRACTID_PREFIX NS_ABOUT_MODULE_CONTRACTID "?what=" 
#define NS_ABOUT_MODULE_CONTRACTID_LENGTH 49      // strlen(NS_ABOUT_MODULE_CONTRACTID_PREFIX)

%}
